// -*- mode: c++; c-file-style: "k&r"; c-basic-offset: 4 -*-
/***********************************************************************
 *
 * memory.h:
 *   parsing and pretty-printing of memory sizes
 *
 * Copyright 2013-2016 Dan R. K. Ports  <drkp@cs.washington.edu>
 * Copyright 2009-2012 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 **********************************************************************/

#ifndef _LIB_MEMORY_H_
#define _LIB_MEMORY_H_

#include <unistd.h>

// Experimentally determined malloc size (for smallish objects, at
// least).  This is (v+8) rounded up to the nearest multiple of 16
// (though anything less than 24 takes 32 bytes).  Obviously this
// doesn't account for fragmentation.
#define MALLOC_SIZE(size) ((size) <= 24 ? 32 : (((size) + 7) | 15) + 1)

#define MEMORY_FMTSIZE_BUF 22

char *Memory_FmtSize(char *buf, size_t n);
size_t Memory_ReadSize(const char *buf, const char **endPtr);

#endif // _LIB_MEMORY_H_
