#include <linux/bpf.h>
#include <linux/if_ether.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/tcp.h>
#include "linux/tools/lib/bpf/bpf_helpers.h"

#include "common.h"

SEC("FastBroadCast")
int FastBroadCast_main(struct __sk_buff *skb)
{ 
    void *data_end = (void *)(long)skb->data_end;
    void *data = (void *)(long)skb->data;
    struct ethhdr *eth = data;
    struct iphdr *ip = data + sizeof(struct ethhdr);
    struct udphdr *udp = data + sizeof(struct ethhdr) + sizeof(struct iphdr);
    char *payload = data + sizeof(struct ethhdr) + sizeof(struct iphdr) + sizeof(struct udphdr); // 指针

    if (ip + 1 > data_end)
        return TC_ACT_OK;
    if (ip->protocol != IPPROTO_UDP)
        return TC_ACT_OK;
    if (udp + 1 > data_end)
        return TC_ACT_OK;
    if (udp->source != htons(12345))
        return TC_ACT_OK; // port暂时为12345

    if (payload + MAGIC_LEN > data_end)
        return TC_ACT_OK; .
    if (payload[0] != 0x18 || payload[1] != 0x03 || payload[2] != 0x05 || payload[3] != 0x20)
        return TC_ACT_OK;
    payload = payload + MAGIC_LEN;
    if (payload + sizeof(__u64) > data_end)
        return TC_ACT_OK;
    __u64 typeLen = *(__u64 *)payload;
    payload = payload + sizeof(__u64);
    char *type_str = payload;
    if (type_str + 5 >= data_end)
        return TC_ACT_OK; 
    if (typeLen >= MTU || payload + typeLen > data_end)
        return TC_ACT_OK; 
    payload += typeLen;
    if (payload + FAST_PAXOS_DATA_LEN > data_end)
        return TC_ACT_OK;

    
    __u32 msg_view = *(__u32 *)payload;
    __u32 is_broadcast = msg_view & BROADCAST_SIGN_BIT;
    msg_view ^= is_broadcast; 
    __u32 msg_lastOp = *((__u32 *)payload + 1);
    int msg_type = compute_message_type(type_str, data_end);

    if (msg_type == FAST_PROG_XDP_HANDLE_PREPARE)
    {                                                      
        __u32 idx = msg_lastOp & (QUORUM_BITSET_ENTRY - 1); 
        struct paxos_quorum *entry = bpf_map_lookup_elem(&map_quorum, &idx);
        if (entry)
        { 
            if (entry->view != msg_view || entry->opnum != msg_lastOp)
            {
                entry->view = msg_view;
                entry->opnum = msg_lastOp;
                entry->bitset = 0;
            }
        }
    }

    if (!is_broadcast)
        return TC_ACT_OK; // 无需优化的情况

    __u32 zero = 0;
    struct paxos_ctr_state *ctr_state = bpf_map_lookup_elem(&map_ctr_state, &zero);
    if (!ctr_state)
        return TC_ACT_OK; 

    char id, nxt; 
    if (type_str[0] == 's' && type_str[1] == 'p')
    {
        id = !ctr_state->leaderIdx; 

        nxt = id + 1;
        nxt += ctr_state->leaderIdx == nxt;
        type_str[0] = nxt;
        type_str[1] = 'M'; // 多播标志
        if (nxt < CLUSTER_SIZE)
            bpf_clone_redirect(skb, skb->ifindex, 0); 
    else
    {
        id = type_str[0];

        nxt = id + 1;
        nxt += ctr_state->leaderIdx == nxt;
        type_str[0] = nxt;
        if (nxt < CLUSTER_SIZE)
            bpf_clone_redirect(skb, skb->ifindex, 0); 
    }

  
    data_end = (void *)(long)skb->data_end;
    data = (void *)(long)skb->data;
    eth = data;
    ip = data + sizeof(struct ethhdr);
    udp = data + sizeof(struct ethhdr) + sizeof(struct iphdr);
    payload = data + sizeof(struct ethhdr) + sizeof(struct iphdr) + sizeof(struct udphdr) + MAGIC_LEN;
    if (payload + sizeof(__u64) > data_end)
        return TC_ACT_OK; 
    typeLen = *(__u64 *)payload;
    payload = payload + sizeof(__u64);
    type_str = payload;
    if (type_str + 5 >= data_end)
        return TC_ACT_SHOT;
    if (typeLen >= MTU || payload + typeLen > data_end)
        return TC_ACT_SHOT; 
    payload += typeLen;
    if (payload + FAST_PAXOS_DATA_LEN > data_end)
        return TC_ACT_SHOT;

    *(__u32 *)payload = msg_view;
    type_str[0] = 's', type_str[1] = 'p';
    struct paxos_configure *replicaInfo = bpf_map_lookup_elem(&map_configure, &id);
    if (!replicaInfo)
        return TC_ACT_SHOT;
    // 改为目标地址
    udp->dest = replicaInfo->port;
    udp->check = 0;
    ip->daddr = replicaInfo->addr;
    ip->check = compute_ip_checksum(ip);
    memcpy(eth->h_dest, replicaInfo->eth, ETH_ALEN);

    return TC_ACT_OK;
    }
}