#include <linux/bpf.h>
#include <linux/if_ether.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/tcp.h>
#include "linux/tools/lib/bpf/bpf_helpers.h"

#include "common.h"


SEC("PrepareFastReply")
int PrepareFastReply_main(struct xdp_md *ctx)
{
	void *data_end = (void *)(long)ctx->data_end;
	void *data = (void *)(long)ctx->data;
	struct ethhdr *eth = data;
	struct iphdr *ip = data + sizeof(struct ethhdr);
	struct udphdr *udp = data + sizeof(struct ethhdr) + sizeof(struct iphdr);
	char *payload = data + sizeof(struct ethhdr) + sizeof(struct iphdr) + sizeof(struct udphdr);

	if (payload + MAGIC_LEN + sizeof(__u64) + PREPARE_TYPE_LEN + FAST_PAXOS_DATA_LEN + sizeof(__u64) >= data_end)
		return XDP_PASS;

	// 查看当前状态
	__u32 zero = 0;
	__u64 *msg_lastOp = bpf_map_lookup_elem(&map_msg_lastOp, &zero);
	struct paxos_ctr_state *ctr_state = bpf_map_lookup_elem(&map_ctr_state, &zero);
	if (!msg_lastOp || !ctr_state)
		return XDP_PASS; 

	struct paxos_configure *leaderInfo = bpf_map_lookup_elem(&map_configure, &ctr_state->leaderIdx);
	if (!leaderInfo)
		return XDP_PASS;

	*(__u32 *)payload = NONFRAG_MAGIC;
	payload += sizeof(__u32);
	*(__u64 *)payload = MYPREPAREOK_TYPE_LEN;
	payload += sizeof(__u64);

	payload[13] = 'M', payload[14] = 'y', payload[15] = 'P', payload[16] = 'r',
	payload[17] = 'e', payload[18] = 'p', payload[19] = 'a', payload[20] = 'r',
	payload[21] = 'e', payload[22] = 'O', payload[23] = 'K';
	payload += MYPREPAREOK_TYPE_LEN;

	*(__u32 *)payload = ctr_state->view;	
	*((__u32 *)payload + 1) = *msg_lastOp;		
	*((__u32 *)payload + 2) = ctr_state->myIdx; 
	payload += FAST_PAXOS_DATA_LEN;
	if (payload + sizeof(__u64) * 3 + sizeof(__u32) > data_end)
	
		return XDP_PASS;
	
	*(__u64 *)payload = sizeof(__u64) * 2 + sizeof(__u32);
	*((__u64 *)payload + 1) = ctr_state->view; 
	*((__u64 *)payload + 2) = *msg_lastOp;	   
	payload += sizeof(__u64) * 3;
	*(__u32 *)payload = ctr_state->myIdx;
	payload += sizeof(__u32);

	udp->source = udp->dest;
	udp->dest = leaderInfo->port;
	udp->len = htons(payload - (char *)udp);
	udp->check = 0;							 // 不必计算udp校验和

	ip->tot_len = htons(payload - (char *)udp + sizeof(struct iphdr));
	ip->saddr = ip->daddr;
	ip->daddr = leaderInfo->addr;
	ip->check = compute_ip_checksum(ip);

	unsigned char tmp_mac[ETH_ALEN];
	memcpy(tmp_mac, eth->h_source, ETH_ALEN);
	memcpy(eth->h_source, eth->h_dest, ETH_ALEN);
	memcpy(eth->h_dest, tmp_mac, ETH_ALEN);

	bpf_xdp_adjust_tail(ctx, (void *)payload - data_end);
	return XDP_TX;
}